@@ -75,7 +75,7 @@ public:
       throw primecount_error("z must be <= FactorTable::max()");
 
     z = std::max<int64_t>(1, z);
-    T T_MAX = port::numeric_limits<T>::max();
+    T T_MAX = pstd::numeric_limits<T>::max();
     factor_.resize(to_index(z) + 1);
 
     // mu(1) = 1.
@@ -224,7 +224,7 @@ public:
 
   static maxint_t max()
   {
-    maxint_t T_MAX = port::numeric_limits<T>::max();
+    maxint_t T_MAX = pstd::numeric_limits<T>::max();
     return ipow<2>(T_MAX - 1) - 1;
   }
 
