@@ -50,18 +50,17 @@
 
 #elif defined(__AVX512F__) && \
       defined(__AVX512VPOPCNTDQ__) && \
-      defined(__BMI2__) && \
-     !defined(__i386__) /* misses _bzhi_u64() */ && \
       __has_include(<immintrin.h>)
   #include <immintrin.h>
-  #define ENABLE_AVX512_BMI2
+  #define ENABLE_AVX512_VPOPCNT
 
 #elif defined(ENABLE_MULTIARCH_ARM_SVE)
   #include <cpu_supports_arm_sve.hpp>
   #include <arm_sve.h>
   #define ENABLE_DEFAULT
-#elif defined(ENABLE_MULTIARCH_AVX512_BMI2)
-  #include <cpu_supports_avx512_bmi2.hpp>
+
+#elif defined(ENABLE_MULTIARCH_AVX512_VPOPCNT)
+  #include <cpu_supports_avx512_vpopcnt.hpp>
   #include <immintrin.h>
   #define ENABLE_DEFAULT
 #else
@@ -78,6 +77,21 @@ public:
   void cross_off_count(uint64_t prime, uint64_t i);
   static uint64_t get_segment_size(uint64_t size);
 
+  uint64_t get_total_count() const
+  {
+    return total_count_;
+  }
+
+  template <typename T>
+  void pre_sieve(const Vector<T>& primes, uint64_t c, uint64_t low, uint64_t high)
+  {
+    reset_sieve(low, high);
+    for (uint64_t i = 4; i <= c; i++)
+      cross_off(primes[i], i);
+
+    init_counter(low, high);
+  }
+
   /// Count 1 bits inside [0, stop]
   ALWAYS_INLINE uint64_t count(uint64_t stop)
   {
@@ -113,32 +127,17 @@ public:
   {
     #if defined(ENABLE_ARM_SVE)
       return count_arm_sve(start, stop);
-    #elif defined(ENABLE_AVX512_BMI2)
-      return count_avx512_bmi2(start, stop);
+    #elif defined(ENABLE_AVX512_VPOPCNT)
+      return count_avx512(start, stop);
     #elif defined(ENABLE_MULTIARCH_ARM_SVE)
       return cpu_supports_sve ? count_arm_sve(start, stop) : count_default(start, stop);
-    #elif defined(ENABLE_MULTIARCH_AVX512_BMI2)
-      return cpu_supports_avx512_bmi2 ? count_avx512_bmi2(start, stop) : count_default(start, stop);
+    #elif defined(ENABLE_MULTIARCH_AVX512_VPOPCNT)
+      return cpu_supports_avx512_vpopcnt ? count_avx512(start, stop) : count_default(start, stop);
     #else
       return count_default(start, stop);
     #endif
   }
 
-  template <typename T>
-  void pre_sieve(const Vector<T>& primes, uint64_t c, uint64_t low, uint64_t high)
-  {
-    reset_sieve(low, high);
-    for (uint64_t i = 4; i <= c; i++)
-      cross_off(primes[i], i);
-
-    init_counter(low, high);
-  }
-
-  uint64_t get_total_count() const
-  {
-    return total_count_;
-  }
-
 private:
 
 #if defined(ENABLE_DEFAULT)
@@ -148,7 +147,7 @@ private:
   /// hence we simply count the number of unsieved elements
   /// by linearly iterating over the sieve array.
   ///
-  uint64_t count_default(uint64_t start, uint64_t stop) const
+  ALWAYS_INLINE uint64_t count_default(uint64_t start, uint64_t stop) const
   {
     if (start > stop)
       return 0;
@@ -158,34 +157,39 @@ private:
     uint64_t stop_idx = stop / 240;
     uint64_t m1 = unset_smaller[start % 240];
     uint64_t m2 = unset_larger[stop % 240];
-    uint64_t* sieve64 = (uint64_t*) sieve_.data();
 
-    if (start_idx == stop_idx)
-      return popcnt64(sieve64[start_idx] & (m1 & m2));
-    else
-    {
-      uint64_t cnt = popcnt64(sieve64[start_idx] & m1);
-      for (uint64_t i = start_idx + 1; i < stop_idx; i++)
-        cnt += popcnt64(sieve64[i]);
-      cnt += popcnt64(sieve64[stop_idx] & m2);
-      return cnt;
-    }
+    // Branchfree bitmask calculation:
+    // m1 = (start_idx != stop_idx) ? m1 : m1 & m2;
+    m1 = (m1 * (start_idx != stop_idx)) | ((m1 & m2) * (start_idx == stop_idx));
+    // m2 = (start_idx != stop_idx) ? m2 : 0;
+    m2 *= (start_idx != stop_idx);
+
+    const uint64_t* sieve64 = (const uint64_t*) sieve_.data();
+    uint64_t start_bits = sieve64[start_idx] & m1;
+    uint64_t stop_bits = sieve64[stop_idx] & m2;
+    uint64_t cnt = popcnt64(start_bits);
+    cnt += popcnt64(stop_bits);
+
+    for (uint64_t i = start_idx + 1; i < stop_idx; i++)
+      cnt += popcnt64(sieve64[i]);
+
+    return cnt;
   }
 
 #endif
 
-#if defined(ENABLE_AVX512_BMI2) || \
-    defined(ENABLE_MULTIARCH_AVX512_BMI2)
+#if defined(ENABLE_AVX512_VPOPCNT) || \
+    defined(ENABLE_MULTIARCH_AVX512_VPOPCNT)
 
   /// Count 1 bits inside [start, stop].
   /// The distance [start, stop] is small here < sqrt(segment_size),
   /// hence we simply count the number of unsieved elements
   /// by linearly iterating over the sieve array.
   ///
-  #if defined(ENABLE_MULTIARCH_AVX512_BMI2)
-    __attribute__ ((target ("avx512f,avx512vpopcntdq,bmi2")))
+  #if defined(ENABLE_MULTIARCH_AVX512_VPOPCNT)
+    __attribute__ ((target ("avx512f,avx512vpopcntdq")))
   #endif
-  uint64_t count_avx512_bmi2(uint64_t start, uint64_t stop) const
+  uint64_t count_avx512(uint64_t start, uint64_t stop) const
   {
     if (start > stop)
       return 0;
@@ -195,32 +199,36 @@ private:
     uint64_t stop_idx = stop / 240;
     uint64_t m1 = unset_smaller[start % 240];
     uint64_t m2 = unset_larger[stop % 240];
-    uint64_t* sieve64 = (uint64_t*) sieve_.data();
 
-    if (start_idx == stop_idx)
-      return popcnt64(sieve64[start_idx] & (m1 & m2));
-    else
+    // Branchfree bitmask calculation:
+    // m1 = (start_idx != stop_idx) ? m1 : m1 & m2;
+    m1 = (m1 * (start_idx != stop_idx)) | ((m1 & m2) * (start_idx == stop_idx));
+    // m2 = (start_idx != stop_idx) ? m2 : 0;
+    m2 *= (start_idx != stop_idx);
+
+    const uint64_t* sieve64 = (const uint64_t*) sieve_.data();
+    uint64_t start_bits = sieve64[start_idx] & m1;
+    uint64_t stop_bits = sieve64[stop_idx] & m2;
+    __m512i vec = _mm512_set_epi64(0, 0, 0, 0, 0, 0, stop_bits, start_bits);
+    __m512i vcnt = _mm512_popcnt_epi64(vec);
+    uint64_t i = start_idx + 1;
+
+    // Compute this for loop using AVX512.
+    // for (i = start_idx + 1; i < stop_idx; i++)
+    //   cnt += popcnt64(sieve64[i]);
+    //
+    for (; i + 8 < stop_idx; i += 8)
     {
-      uint64_t i = start_idx + 1;
-      uint64_t cnt = popcnt64(sieve64[start_idx] & m1);
-      __m512i vcnt = _mm512_setzero_si512();
-
-      // Compute this for loop using AVX512.
-      // for (i = start_idx + 1; i < stop_idx; i++)
-      //   cnt += popcnt64(sieve64[i]);
-      do {
-        __mmask8 mask = (i + 8 < stop_idx) ? 0xff : (__mmask8) _bzhi_u64(0xff, stop_idx - i);
-        __m512i vec = _mm512_maskz_loadu_epi64(mask , &sieve64[i]);
-        vec = _mm512_popcnt_epi64(vec);
-        vcnt = _mm512_add_epi64(vcnt, vec);
-        i += 8;
-      }
-      while (i < stop_idx);
-
-      cnt += _mm512_reduce_add_epi64(vcnt);
-      cnt += popcnt64(sieve64[stop_idx] & m2);
-      return cnt;
+      vec = _mm512_loadu_epi64(&sieve64[i]);
+      vec = _mm512_popcnt_epi64(vec);
+      vcnt = _mm512_add_epi64(vcnt, vec);
     }
+
+    __mmask8 mask = (__mmask8) (0xff >> (i + 8 - stop_idx));
+    vec = _mm512_maskz_loadu_epi64(mask, &sieve64[i]);
+    vec = _mm512_popcnt_epi64(vec);
+    vcnt = _mm512_add_epi64(vcnt, vec);
+    return _mm512_reduce_add_epi64(vcnt);
   }
 
 #elif defined(ENABLE_ARM_SVE) || \
@@ -244,33 +252,36 @@ private:
     uint64_t stop_idx = stop / 240;
     uint64_t m1 = unset_smaller[start % 240];
     uint64_t m2 = unset_larger[stop % 240];
-    uint64_t* sieve64 = (uint64_t*) sieve_.data();
 
-    if (start_idx == stop_idx)
-      return popcnt64(sieve64[start_idx] & (m1 & m2));
-    else
-    {
-      uint64_t i = start_idx + 1;
-      uint64_t cnt = popcnt64(sieve64[start_idx] & m1);
-      svbool_t pg = svwhilelt_b64(i, stop_idx);
-      svuint64_t vcnt = svdup_u64(0);
+    // Branchfree bitmask calculation:
+    // m1 = (start_idx != stop_idx) ? m1 : m1 & m2;
+    m1 = (m1 * (start_idx != stop_idx)) | ((m1 & m2) * (start_idx == stop_idx));
+    // m2 = (start_idx != stop_idx) ? m2 : 0;
+    m2 *= (start_idx != stop_idx);
 
-      // Compute this for loop using ARM SVE.
-      // for (i = start_idx + 1; i < stop_idx; i++)
-      //   cnt += popcnt64(sieve64[i]);
-      do {
-        svuint64_t vec = svld1_u64(pg, &sieve64[i]);
-        vec = svcnt_u64_z(pg, vec);
-        vcnt = svadd_u64_z(svptrue_b64(), vcnt, vec);
-        i += svcntd();
-        pg = svwhilelt_b64(i, stop_idx);
-      }
-      while (svptest_any(svptrue_b64(), pg));
+    const uint64_t* sieve64 = (const uint64_t*) sieve_.data();
+    uint64_t start_bits = sieve64[start_idx] & m1;
+    uint64_t stop_bits = sieve64[stop_idx] & m2;
+    uint64_t bounds[2] = { start_bits, stop_bits };
+    ASSERT(svcntd() >= 2);
+    svbool_t pg = svwhilelt_b64(0, 2);
+    svuint64_t vec = svld1_u64(pg, &bounds[0]);
+    svuint64_t vcnt = svcnt_u64_z(pg, vec);
+    uint64_t i = start_idx + 1;
 
-      cnt += svaddv_u64(svptrue_b64(), vcnt);
-      cnt += popcnt64(sieve64[stop_idx] & m2);
-      return cnt;
+    // Compute this for loop using ARM SVE.
+    // for (i = start_idx + 1; i < stop_idx; i++)
+    //   cnt += popcnt64(sieve64[i]);
+    do {
+      pg = svwhilelt_b64(i, stop_idx);
+      vec = svld1_u64(pg, &sieve64[i]);
+      vec = svcnt_u64_z(pg, vec);
+      vcnt = svadd_u64_x(svptrue_b64(), vcnt, vec);
+      i += svcntd();
     }
+    while (i < stop_idx);
+
+    return svaddv_u64(svptrue_b64(), vcnt);
   }
 
 #endif
