@@ -18,7 +18,7 @@
 ///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
 ///        pp. 759-768.
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -29,8 +29,8 @@
 #include <FactorTable.hpp>
 #include <Sieve.hpp>
 #include <fast_div.hpp>
-#include <generate.hpp>
-#include <generate_phi.hpp>
+#include <generate_primes.hpp>
+#include <phi_vector.hpp>
 #include <imath.hpp>
 #include <int128_t.hpp>
 #include <LoadBalancerS2.hpp>
@@ -83,7 +83,7 @@ T S2_hard_thread(T x,
   if (min_b > max_b)
     return 0;
 
-  auto phi = generate_phi(low, max_b, primes, pi);
+  Vector<int64_t> phi = phi_vector(low, max_b, primes, pi);
   Sieve sieve(low, segment_size, max_b);
   thread.init_finished();
 
@@ -216,7 +216,7 @@ T S2_hard_OpenMP(T x,
     {
       // Unsigned integer division is usually slightly
       // faster than signed integer division
-      using UT = typename std::make_unsigned<T>::type;
+      using UT = typename pstd::make_unsigned<T>::type;
 
       thread.start_time();
       UT sum = S2_hard_thread((UT) x, y, z, c, primes, pi, factor, thread);
@@ -254,7 +254,7 @@ int64_t S2_hard(int64_t x,
 
   FactorTable<uint16_t> factor(y, threads);
   int64_t max_prime = min(y, z / isqrt(y));
-  auto primes = generate_primes<int32_t>(max_prime);
+  auto primes = generate_primes<uint32_t>(max_prime);
   int64_t sum = S2_hard_OpenMP(x, y, z, c, s2_hard_approx, primes, factor, threads, is_print);
 
   if (is_print)
