@@ -19,7 +19,7 @@
 ///        In-depth description of this algorithm:
 ///        https://github.com/kimwalisch/primecount/blob/master/doc/Easy-Special-Leaves.md
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -30,7 +30,7 @@
 #include <primecount-internal.hpp>
 #include <LoadBalancerAC.hpp>
 #include <fast_div.hpp>
-#include <generate.hpp>
+#include <generate_primes.hpp>
 #include <gourdon.hpp>
 #include <int128_t.hpp>
 #include <min.hpp>
@@ -40,7 +40,6 @@
 
 #include <stdint.h>
 
-using std::numeric_limits;
 using namespace primecount;
 
 namespace {
@@ -218,7 +217,7 @@ T AC_OpenMP(T x,
   // dual-socket AMD EPYC 7642 server with 192 CPU cores.
   int64_t thread_threshold = 1000;
   int max_threads = (int) std::pow(xz, 1 / 3.7);
-  threads = std::min(threads, max_threads);
+  threads = min(threads, max_threads);
   threads = ideal_num_threads(x13, threads, thread_threshold);
   LoadBalancerAC loadBalancer(sqrtx, y, threads, is_print);
 
@@ -244,13 +243,6 @@ T AC_OpenMP(T x,
   //
   #pragma omp parallel num_threads(threads) reduction(+: sum)
   {
-    // SegmentedPiTable is accessed very frequently.
-    // In order to get good performance it is important that
-    // SegmentedPiTable fits into the CPU's cache.
-    // Hence we use a small segment_size of x^(1/4).
-    SegmentedPiTable segmentedPi;
-    int64_t low, high;
-
     // C1 formula: pi[(x/z)^(1/3)] < b <= pi[pi_sqrtz]
     // There are very few iterations in this loop,
     // hence the use of an atomic loop counter (min_c1)
@@ -266,38 +258,63 @@ T AC_OpenMP(T x,
       sum -= C1<-1>(xp, b, b, pi_y, 1, min_m, max_m, primes, pi);
     }
 
-    // for (low = 0; low < sqrt; low += segment_size)
-    while (loadBalancer.get_work(low, high))
+    // SegmentedPiTable is accessed very frequently.
+    // In order to get good performance it is important that
+    // SegmentedPiTable fits into the CPU's cache.
+    // Hence we use a small segment_size of x^(1/4).
+    SegmentedPiTable segmentedPi;
+    ThreadDataAC thread;
+
+    // for (low = 0; low < sqrt(x); low += segment_size)
+    while (loadBalancer.get_work(thread))
     {
-      // Current segment [low, high[
-      segmentedPi.init(low, high);
-      T xlow = x / max(low, 1);
-      T xhigh = x / high;
+      int64_t low = thread.low;
+      int64_t segment_size = thread.segment_size;
+      int64_t limit = low + thread.segments * segment_size;
+      limit = min(limit, sqrtx);
 
-      int64_t min_c2 = max(k, pi_root3_xy);
-      min_c2 = max(min_c2, pi_sqrtz);
-      min_c2 = max(min_c2, pi[isqrt(low)]);
-      min_c2 = max(min_c2, pi[min(xhigh / y, x_star)]);
-      min_c2 += 1;
+      for (; low < limit; low += segment_size)
+      {
+        // Current segment [low, high[
+        int64_t high = low + segment_size;
+        high = min(high, sqrtx);
+        segmentedPi.init(low, high);
 
-      int64_t min_a = min(xhigh / high, x13);
-      min_a = pi[max(x_star, min_a)] + 1;
-    
-      // Upper bound of A & C2 formulas:
-      // x / (p * q) >= low
-      // p * next_prime(p) <= x / low
-      // p <= sqrt(x / low)
-      T sqrt_xlow = isqrt(xlow);
-      int64_t max_c2 = pi[min(sqrt_xlow, x_star)];
-      int64_t max_a = pi[min(sqrt_xlow, x13)];
+        // We measure the thread computation time excluding the
+        // first expensive initialization of the segmentedPi
+        // lookup table. If the thread computation time is close
+        // to 0 then we increase the number of segments in the
+        // loadBalancer which should improve performance.
+        if (low == thread.low)
+          thread.secs = get_time();
 
-      // C2 formula: pi[sqrt(z)] < b <= pi[x_star]
-      for (int64_t b = min_c2; b <= max_c2; b++)
-        sum += C2(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+        T xlow = x / max(low, 1);
+        T xhigh = x / high;
+        int64_t min_c2 = max(k, pi_root3_xy);
+        min_c2 = max(min_c2, pi_sqrtz);
+        min_c2 = max(min_c2, pi[isqrt(low)]);
+        min_c2 = max(min_c2, pi[min(xhigh / y, x_star)]);
+        min_c2 += 1;
 
-      // A formula: pi[x_star] < b <= pi[x13]
-      for (int64_t b = min_a; b <= max_a; b++)
-        sum += A(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+        int64_t min_a = min(xhigh / high, x13);
+        min_a = pi[max(x_star, min_a)] + 1;
+      
+        // Upper bound of A & C2 formulas:
+        // x / (p * q) >= low
+        // p * next_prime(p) <= x / low
+        // p <= sqrt(x / low)
+        T sqrt_xlow = isqrt(xlow);
+        int64_t max_c2 = pi[min(sqrt_xlow, x_star)];
+        int64_t max_a = pi[min(sqrt_xlow, x13)];
+
+        // C2 formula: pi[sqrt(z)] < b <= pi[x_star]
+        for (int64_t b = min_c2; b <= max_c2; b++)
+          sum += C2(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+
+        // A formula: pi[x_star] < b <= pi[x13]
+        for (int64_t b = min_a; b <= max_a; b++)
+          sum += A(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+      }
     }
   }
 
@@ -365,7 +382,7 @@ int128_t AC(int128_t x,
   int128_t sum;
 
   // uses less memory
-  if (max_prime <= numeric_limits<uint32_t>::max())
+  if (max_prime <= pstd::numeric_limits<uint32_t>::max())
   {
     auto primes = generate_primes<uint32_t>(max_prime);
     sum = AC_OpenMP((uint128_t) x, y, z, k, x_star, max_a_prime, primes, threads, is_print);
