@@ -15,6 +15,7 @@
 #include <int128_t.hpp>
 #include <imath.hpp>
 #include <macros.hpp>
+#include <min.hpp>
 
 #include <algorithm>
 #include <chrono>
@@ -23,9 +24,6 @@
 #include <stdint.h>
 #include <utility>
 
-using std::min;
-using std::max;
-
 namespace {
 
 int status_precision_ = -1;
@@ -113,7 +111,7 @@ maxint_t to_maxint(const std::string& expr)
     if (pos != std::string::npos)
     {
       std::string n = expr.substr(pos);
-      maxint_t limit = port::numeric_limits<maxint_t>::max();
+      maxint_t limit = pstd::numeric_limits<maxint_t>::max();
       std::string max_n = to_string(limit);
 
       if (n.size() > max_n.size() ||
@@ -432,7 +430,7 @@ int64_t get_x_star_gourdon(maxint_t x, int64_t y)
   // numbers below 10^6.
   x_star = min(x_star, y);
   x_star = min(x_star, sqrt_xy);
-  x_star = max(x_star, (int64_t) 1);
+  x_star = max(x_star, 1);
 
   return x_star;
 }
